@@ -1,5 +1,4 @@
 # Imports.
-from torch.utils.tensorboard import SummaryWriter
 import torch
 import numpy as np
 from typing import Callable, Tuple
@@ -12,9 +11,9 @@ def wasserstein_singular_vectors(
     dtype: torch.dtype,
     device: str,
     n_iter: int,
-    tau: float = 1e-3,
+    tau: float = 0,
     p: int = 1,
-    writer: SummaryWriter = None,
+    writer=None,
     small_value: float = 1e-6,
     normalization_steps: int = 1,
     C_ref: torch.Tensor = None,
@@ -25,11 +24,11 @@ def wasserstein_singular_vectors(
     """Performs power iterations and returns Wasserstein Singular Vectors. Early stopping is possible with Ctrl-C.
 
     Args:
-        dataset (torch.Tensor): The input dataset, rows as samples
+        dataset (torch.Tensor): The input dataset, rows as samples. Alternatively, you can give a tuple of tensors (A, B).
         dtype (str): The dtype
         device (str): The device
         n_iter (int): The number of power iterations.
-        tau (float, optional): The regularization parameter for the norm R. Defaults to 1e-3.
+        tau (float, optional): The regularization parameter for the norm R. Defaults to 0.
         p (int, optional): The order of the norm R. Defaults to 1.
         writer (SummaryWriter, optional): If set, the progress will be written to the Tensorboard writer. Defaults to None.
         small_value (float, optional): A small value for numerical stability. Defaults to 1e-6.
@@ -44,22 +43,32 @@ def wasserstein_singular_vectors(
     """
 
     # Perform some sanity checks.
-    assert len(dataset.shape) == 2 # correct shape
-    assert torch.sum(dataset < 0) == 0 # positivity
-    assert n_iter > 0 # at least one iteration
-    assert tau >= 0 # a positive regularization
-    assert p > 0 # a valid norm
-    assert small_value > 0 # a positive numerical offset
-    assert normalization_steps > 0 # normalizing at least once
+    assert n_iter > 0  # at least one iteration
+    assert tau >= 0  # a positive regularization
+    assert p > 0  # a valid norm
+    assert small_value > 0  # a positive numerical offset
+    assert normalization_steps > 0  # normalizing at least once
 
-    # Make the transposed datasets A and B from the dataset.
-    A, B = utils.normalize_dataset(
-        dataset,
-        normalization_steps=normalization_steps,
-        small_value=small_value,
-        dtype=dtype,
-        device=device,
-    )
+    if type(dataset) is tuple:
+        assert len(dataset) == 2  # correct shape
+
+        A, B = dataset  # Recover A and B
+
+        assert torch.sum(A < 0) == 0  # positivity
+        assert torch.sum(B < 0) == 0  # positivity
+
+    else:
+        assert len(dataset.shape) == 2  # correct shape
+        assert torch.sum(dataset < 0) == 0  # positivity
+
+        # Make the transposed datasets A and B from the dataset.
+        A, B = utils.normalize_dataset(
+            dataset,
+            normalization_steps=normalization_steps,
+            small_value=small_value,
+            dtype=dtype,
+            device=device,
+        )
 
     # Compute the regularization matrices.
     R_A = utils.regularization_matrix(A, p=p, dtype=dtype, device=device)
@@ -80,7 +89,7 @@ def wasserstein_singular_vectors(
             D_new = distance.wasserstein_map(
                 A,
                 C,
-                R_A,
+                R=R_A,
                 tau=tau,
                 dtype=dtype,
                 device=device,
@@ -105,7 +114,7 @@ def wasserstein_singular_vectors(
             C_new = distance.wasserstein_map(
                 B,
                 D,
-                R_B,
+                R=R_B,
                 tau=tau,
                 dtype=dtype,
                 device=device,
@@ -143,10 +152,10 @@ def sinkhorn_singular_vectors(
     dtype: str,
     device: str,
     n_iter: int,
-    tau: float = 1e-3,
+    tau: float = 0,
     eps: float = 5e-2,
     p: int = 1,
-    writer: SummaryWriter = None,
+    writer=None,
     small_value: float = 1e-6,
     normalization_steps: int = 1,
     C_ref: torch.Tensor = None,
@@ -157,11 +166,11 @@ def sinkhorn_singular_vectors(
     """Performs power iterations and returns Sinkhorn Singular Vectors. Early stopping is possible with Ctrl-C.
 
     Args:
-        dataset (torch.Tensor): The input dataset
+        dataset (torch.Tensor): The input dataset. Alternatively, you can give a tuple of tensors (A, B).
         dtype (str): The dtype
         device (str): The device
         n_iter (int): The number of power iterations.
-        tau (float, optional): The regularization parameter for the norm R. Defaults to 1e-3.
+        tau (float, optional): The regularization parameter for the norm R. Defaults to 0.
         eps (float): The entropic regularization parameter.
         p (int, optional): The order of the norm R. Defaults to 1.
         writer (SummaryWriter, optional): If set, the progress will be written to the Tensorboard writer. Defaults to None.
@@ -177,23 +186,33 @@ def sinkhorn_singular_vectors(
     """
 
     # Perform some sanity checks.
-    assert len(dataset.shape) == 2 # correct shape
-    assert torch.sum(dataset < 0) == 0 # positivity
-    assert n_iter > 0 # at least one iteration
-    assert tau >= 0 # a positive regularization
-    assert eps >= 0 # a positive entropic regularization
-    assert p > 0 # a valid norm
-    assert small_value > 0 # a positive numerical offset
-    assert normalization_steps > 0 # normalizing at least once
+    assert n_iter > 0  # at least one iteration
+    assert tau >= 0  # a positive regularization
+    assert eps >= 0  # a positive entropic regularization
+    assert p > 0  # a valid norm
+    assert small_value > 0  # a positive numerical offset
+    assert normalization_steps > 0  # normalizing at least once
 
-    # Make the transposed datasets A and B from the dataset U.
-    A, B = utils.normalize_dataset(
-        dataset,
-        normalization_steps=normalization_steps,
-        small_value=small_value,
-        dtype=dtype,
-        device=device,
-    )
+    if type(dataset) is tuple:
+        assert len(dataset) == 2  # correct shape
+
+        A, B = dataset  # Recover A and B
+
+        assert torch.sum(A < 0) == 0  # positivity
+        assert torch.sum(B < 0) == 0  # positivity
+
+    else:
+        assert len(dataset.shape) == 2  # correct shape
+        assert torch.sum(dataset < 0) == 0  # positivity
+
+        # Make the transposed datasets A and B from the dataset.
+        A, B = utils.normalize_dataset(
+            dataset,
+            normalization_steps=normalization_steps,
+            small_value=small_value,
+            dtype=dtype,
+            device=device,
+        )
 
     # Compute the regularization matrices.
     R_A = utils.regularization_matrix(A, p=p, dtype=dtype, device=device)
@@ -215,7 +234,7 @@ def sinkhorn_singular_vectors(
             D_new = distance.sinkhorn_map(
                 A,
                 C,
-                R_A,
+                R=R_A,
                 tau=tau,
                 eps=eps,
                 dtype=dtype,
@@ -239,7 +258,7 @@ def sinkhorn_singular_vectors(
             C_new = distance.sinkhorn_map(
                 B,
                 D,
-                R_B,
+                R=R_B,
                 tau=tau,
                 eps=eps,
                 dtype=dtype,
@@ -278,12 +297,12 @@ def stochastic_wasserstein_singular_vectors(
     dtype: torch.dtype,
     device: str,
     n_iter: int,
-    tau: float = 1e-3,
+    tau: float = 0,
     sample_prop: float = 1e-1,
     p: int = 1,
     step_fn: Callable = lambda k: 1 / np.sqrt(k),
     mult_update: bool = False,
-    writer: SummaryWriter = None,
+    writer=None,
     small_value: float = 1e-6,
     normalization_steps: int = 1,
     C_ref: torch.Tensor = None,
@@ -293,11 +312,11 @@ def stochastic_wasserstein_singular_vectors(
     """Performs stochastic power iterations and returns Wasserstein Singular Vectors. Early stopping is possible with Ctrl-C.
 
     Args:
-        dataset (torch.Tensor): The input dataset
+        dataset (torch.Tensor): The input dataset.  Alternatively, you can give a tuple of tensors (A, B).
         dtype (torch.dtype): The dtype
         device (str): The device
         n_iter (int): The number of power iterations.
-        tau (float, optional): The regularization parameter for the norm R. Defaults to 1e-3.
+        tau (float, optional): The regularization parameter for the norm R. Defaults to 0.
         sample_prop (float, optional): The proportion of indices to update at each step. Defaults to 1e-1.
         p (int, optional): The order of the norm R. Defaults to 1.
         step_fn (Callable, optional): The function that defines step size from the iteration number (which starts at 1). Defaults to lambdak:1/np.sqrt(k).
@@ -314,23 +333,33 @@ def stochastic_wasserstein_singular_vectors(
     """
 
     # Perform some sanity checks.
-    assert len(dataset.shape) == 2 # correct shape
-    assert torch.sum(dataset < 0) == 0 # positivity
-    assert n_iter > 0 # at least one iteration
-    assert tau >= 0 # a positive regularization
-    assert 0 < sample_prop <= 1 # a valid proportion
-    assert p > 0 # a valid norm
-    assert small_value > 0 # a positive numerical offset
-    assert normalization_steps > 0 # normalizing at least once
+    assert n_iter > 0  # at least one iteration
+    assert tau >= 0  # a positive regularization
+    assert 0 < sample_prop <= 1  # a valid proportion
+    assert p > 0  # a valid norm
+    assert small_value > 0  # a positive numerical offset
+    assert normalization_steps > 0  # normalizing at least once
 
-    # Make the transposed datasets A and B from the dataset U.
-    A, B = utils.normalize_dataset(
-        dataset,
-        normalization_steps=normalization_steps,
-        small_value=small_value,
-        dtype=dtype,
-        device=device,
-    )
+    if type(dataset) is tuple:
+        assert len(dataset) == 2  # correct shape
+
+        A, B = dataset  # Recover A and B
+
+        assert torch.sum(A < 0) == 0  # positivity
+        assert torch.sum(B < 0) == 0  # positivity
+
+    else:
+        assert len(dataset.shape) == 2  # correct shape
+        assert torch.sum(dataset < 0) == 0  # positivity
+
+        # Make the transposed datasets A and B from the dataset.
+        A, B = utils.normalize_dataset(
+            dataset,
+            normalization_steps=normalization_steps,
+            small_value=small_value,
+            dtype=dtype,
+            device=device,
+        )
 
     # Compute the regularization matrices.
     R_A = utils.regularization_matrix(A, p=p, dtype=dtype, device=device)
@@ -479,13 +508,13 @@ def stochastic_sinkhorn_singular_vectors(
     dtype: torch.dtype,
     device: str,
     n_iter: int,
-    tau: float = 1e-3,
+    tau: float = 0,
     eps: float = 5e-2,
     sample_prop: float = 1e-1,
     p: int = 1,
     step_fn: Callable = lambda k: 1 / np.sqrt(k),
     mult_update: bool = False,
-    writer: SummaryWriter = None,
+    writer=None,
     small_value: float = 1e-6,
     normalization_steps: int = 1,
     C_ref: torch.Tensor = None,
@@ -495,11 +524,11 @@ def stochastic_sinkhorn_singular_vectors(
     """Performs stochastic power iterations and returns Sinkhorn Singular Vectors. Early stopping is possible with Ctrl-C.
 
     Args:
-        dataset (torch.Tensor): The input dataset
+        dataset (torch.Tensor): The input dataset.  Alternatively, you can give a tuple of tensors (A, B).
         dtype (torch.dtype): The dtype
         device (str): The device
         n_iter (int): The number of power iterations.
-        tau (float, optional): The regularization parameter for the norm R. Defaults to 1e-3.
+        tau (float, optional): The regularization parameter for the norm R. Defaults to 0.
         eps (float, optional): The entropic regularization parameter. Defaults to 5e-2.
         sample_prop (float, optional): The proportion of indices to update at each step. Defaults to 1e-1.
         p (int, optional): The order of the norm R. Defaults to 1.
@@ -517,24 +546,34 @@ def stochastic_sinkhorn_singular_vectors(
     """
 
     # Perform some sanity checks.
-    assert len(dataset.shape) == 2 # correct shape
-    assert torch.sum(dataset < 0) == 0 # positivity
-    assert n_iter > 0 # at least one iteration
-    assert tau >= 0 # a positive regularization
-    assert 0 < sample_prop <= 1 # a valid proportion
-    assert eps >= 0 # a positive entropic regularization
-    assert p > 0 # a valid norm
-    assert small_value > 0 # a positive numerical offset
-    assert normalization_steps > 0 # normalizing at least once
+    assert n_iter > 0  # at least one iteration
+    assert tau >= 0  # a positive regularization
+    assert 0 < sample_prop <= 1  # a valid proportion
+    assert eps >= 0  # a positive entropic regularization
+    assert p > 0  # a valid norm
+    assert small_value > 0  # a positive numerical offset
+    assert normalization_steps > 0  # normalizing at least once
 
-    # Make the transposed datasets A and B from the dataset U.
-    A, B = utils.normalize_dataset(
-        dataset,
-        normalization_steps=normalization_steps,
-        small_value=small_value,
-        dtype=dtype,
-        device=device,
-    )
+    if type(dataset) is tuple:
+        assert len(dataset) == 2  # correct shape
+
+        A, B = dataset  # Recover A and B
+
+        assert torch.sum(A < 0) == 0  # positivity
+        assert torch.sum(B < 0) == 0  # positivity
+
+    else:
+        assert len(dataset.shape) == 2  # correct shape
+        assert torch.sum(dataset < 0) == 0  # positivity
+
+        # Make the transposed datasets A and B from the dataset.
+        A, B = utils.normalize_dataset(
+            dataset,
+            normalization_steps=normalization_steps,
+            small_value=small_value,
+            dtype=dtype,
+            device=device,
+        )
 
     # Compute the regularization matrices.
     R_A = utils.regularization_matrix(A, p=p, dtype=dtype, device=device)
